/*
 * TestListener.java
 *  
 * Copyright 2008 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jmunit.anttask;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.tools.ant.BuildEvent;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.BuildListener;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;

/**
 * The TestListener task see whether the a JMUnit failure or error is written to
 * the standard output by an Ant task.
 * 
 * This class is a modification of the Hammock TestListener Ant task.
 * 
 * @author C.A. Meijer
 * @since JMUnit 1.2.
 */
public class TestListener extends Task implements BuildListener {

	private static boolean isRunning = false;

	private boolean runSet;

	private static boolean haltonerror = false;

	private static boolean haltonfailure = false;

	private static String failureproperty;

	private static String errorproperty;

	private static String todir = ".";

	private static String xmlReport;

	private static TestListener instance = new TestListener();
	
	private static boolean collectMessages;

	public void buildFinished(BuildEvent arg0) {
	}

	public void buildStarted(BuildEvent arg0) {
	}

	public void messageLogged(BuildEvent be) {
		if (be.getPriority() <= Project.MSG_INFO) {
			String message = be.getMessage();
			message = message.trim();
			if (message.startsWith("<XmlTestResultsListener>")) {
				collectMessages = true;
			} else if (message.startsWith("</XmlTestResultsListener>")) {
				collectMessages = false;
			} else if (collectMessages){
				xmlReport += message + "\n";
			}
		}
	}

	public void targetFinished(BuildEvent be) {
	}

	public void targetStarted(BuildEvent be) {
	}

	public void taskFinished(BuildEvent be) {
	}

	public void taskStarted(BuildEvent be) {
	}

	public void setTargettask(String taskname) {
	}

	public void setRun(boolean b) {
		if (isRunning == b) {
			throw new BuildException("The running status has already been "
					+ ((b == true) ? "set." : "cleared."));
		}
		this.runSet = true;
		isRunning = b;
		if (b == false) {
			this.getProject().removeBuildListener(instance);
		} else {
			this.getProject().addBuildListener(instance);
			xmlReport = "";
		}
	}

	public void setHaltonerror(boolean b) {
		haltonerror = b;
	}

	public void setHaltonfailure(boolean b) {
		haltonfailure = b;
	}

	public void setFailureproperty(String failureproperty) {
		TestListener.failureproperty = failureproperty;
	}

	public void setErrorproperty(String errorproperty) {
		TestListener.errorproperty = errorproperty;
	}

	public void setTodir(String todir) {
		TestListener.todir = todir;
	}

	private void processError() {
		if (haltonerror) {
			throw new BuildException("Test reported an error.");
		}
		if (errorproperty != null) {
			Project proj = getProject();
			proj.setProperty(errorproperty, "true");
		}
	}

	private void processFailure() {
		if (haltonfailure) {
			throw new BuildException("Test reported a failure.");
		}
		if (failureproperty != null) {
			Project proj = getProject();
			proj.setProperty(failureproperty, "true");
		}
	}

	private void processResults() {
		FileWriter writer;
		try {
			writer = new FileWriter(todir + File.separator
					+ "TEST-AllTests.xml");
			writer.write(xmlReport);
			writer.close();
		} catch (IOException e) {
			throw new BuildException(e);
		}
		if (xmlReport.indexOf("<error") >= 0) {
			processError();
			processFailure();
		}
		if (xmlReport.indexOf("<failure") >= 0) {
			processFailure();
		}
	}

	public void execute() {
		if (this.runSet != true) {
			throw new BuildException(
					"You must start or stop this task using the \"run\""
							+ " attribute.");
		}
		if (!isRunning) {
			processResults();
		}
	}
}
