package jmunit.framework.cldc10;

import j2meunit.framework.Test;
import j2meunit.framework.TestCase;
import j2meunit.framework.TestSuite;
import jmunit.framework.cldc10.mocks.MockTest;
import jmunit.framework.cldc10.mocks.MockTestResult;

import com.hammingweight.hammock.Hammock;

public class TestSuiteTest extends TestCase {

	// The OUT.
	private jmunit.framework.cldc10.TestSuite testSuite;

	// The mocks.
	private Hammock hammock;

	private MockTest mockTest;

	private MockTestResult mockResult;

	public void setUp() {
		// Create the OUT.
		this.testSuite = AllTestsRunner.testSuite;
		
		// Create the mocks.
		this.hammock = new Hammock();
		this.mockTest = AllTestsRunner.mockTest2;
		this.mockTest.setInvocationHandler(this.hammock);
		this.mockResult = new MockTestResult(this.hammock);
	}

	public void tearDown() {
		this.testSuite.removeAll();
	}

	// We should have zero tests in the TestSuite initially.
	public void testCountTestCases() {
		assertEquals(0, this.testSuite.countTestCases());
	}

	// Test that TestSuite probes its children to determine how many tests there
	// are.
	public void testCountChildTests() {
		this.testSuite.add(AllTestsRunner.mockTest);
		assertEquals(3, this.testSuite.countTestCases());
		this.testSuite.add(AllTestsRunner.mockTest);
		assertEquals(6, this.testSuite.countTestCases());
	}

	// Test that running a TestSuite runs all the children.
	public void testRun() {
		// Expectations.
		this.hammock.setExpectation(MockTest.MTHD_RUN_$_TESTRESULT,
				new Object[] { this.mockResult }).setNumberInvocations(3);

		// Exercise.
		this.testSuite.add(this.mockTest);
		this.testSuite.add(this.mockTest);
		this.testSuite.add(this.mockTest);
		this.testSuite.run(this.mockResult);

		// Verify.
		this.hammock.verify();
	}

	public void runTest() throws Throwable {
		if (getName().equals("testCountTestCases")) {
			testCountTestCases();
		} else if (getName().equals("testCountChildTests")) {
			testCountChildTests();
		} else if (getName().equals("testRun")) {
			testRun();
		} else {
			fail("Unknown test: " + getName());
		}
	}

	public Test suite() {
		TestSuite suite = new TestSuite(this.getClass(), new String[] {
				"testCountTestCases", "testCountChildTests", "testRun" });
		return suite;
	}

}
