package jmunit.framework.cldc11;

import j2meunit.framework.Test;
import j2meunit.framework.TestCase;
import j2meunit.framework.TestSuite;

/**
 * This class tests some of the assert and fail methods in the Assertion class.
 * 
 * @author C.A. Meijer
 * 
 */
public class AssertionTest extends TestCase {

	// Tests the fail method with no message
	public void testFail() throws Throwable {
		try {
			Assertion.fail();
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
			assertNull(afe.getMessage());
		}
	}

	// Tests the fail method with a message.
	public void testFailMessage() throws Throwable {
		try {
			Assertion.fail("foo");
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
			assertEquals("foo", afe.getMessage());
		}
	}

	// Tests fail with booleans.
	public void testFailBoolean() throws Throwable {
		try {
			Assertion.fail("foo", false, true);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test assertEquals with Strings
	public void testAssertEqualsStrings() throws Throwable {
		Assertion.assertEquals("foo", "abcd", "abcd");
		try {
			Assertion.assertEquals("foo", "abcde", "abcd");
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test assertEquals with booleans.
	public void testAssertEqualsBoolean() throws Throwable {
		Assertion.assertEquals(false, false);
		Assertion.assertEquals(true, true);
		Assertion.assertNotEquals(false, true);
		try {
			Assertion.assertEquals(false, true);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test AssertEquals with floats.
	public void testAssertEqualsFloat() throws Throwable {
		Assertion.assertEquals(1.0f, 1.0f);
		Assertion.assertNotEquals(1.0f, 2.0f);
		try {
			Assertion.assertEquals(1.0f, 2.0f);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test AssertEquals with doubles.
	public void testAssertEqualsDouble() throws Throwable {
		Assertion.assertEquals(1.0, 1.0);
		Assertion.assertNotEquals(1.0, 2.0);
		try {
			Assertion.assertEquals(1.0, 2.0);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test AssertEquals with bytes.
	public void testAssertEqualsBytes() throws Throwable {
		byte b1 = 1;
		byte b2 = 2;

		Assertion.assertEquals(b1, b1);
		Assertion.assertNotEquals(b1, b2);
		try {
			Assertion.assertEquals(b1, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
		try {
			Assertion.assertNotEquals(b2, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test AssertEquals with shorts
	public void testAssertEqualsShorts() throws Throwable {
		short b1 = 1;
		short b2 = 2;

		Assertion.assertEquals(b1, b1);
		Assertion.assertNotEquals(b1, b2);
		try {
			Assertion.assertEquals(b1, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
		try {
			Assertion.assertNotEquals(b2, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test AssertEquals with ints
	public void testAssertEqualsInts() throws Throwable {
		int b1 = 1;
		int b2 = 2;

		Assertion.assertEquals(b1, b1);
		Assertion.assertNotEquals(b1, b2);
		try {
			Assertion.assertEquals(b1, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
		try {
			Assertion.assertNotEquals(b2, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test AssertEquals with longs
	public void testAssertEqualsLongs() throws Throwable {
		long b1 = 0x789abcdefL;
		long b2 = 0x123456789abcdefL;

		Assertion.assertEquals(b1, b1);
		Assertion.assertNotEquals(b1, b2);
		try {
			Assertion.assertEquals(b1, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
		try {
			Assertion.assertNotEquals(b2, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test AssertEquals with Objects
	public void testAssertEqualsObjects() throws Throwable {
		Integer b1 = new Integer(1);
		Integer b2 = new Integer(2);

		Assertion.assertEquals(b1, b1);
		Assertion.assertNotEquals(b1, b2);
		try {
			Assertion.assertEquals(b1, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
		try {
			Assertion.assertNotEquals(b2, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test AssertSame
	public void testAssertSame() throws Throwable {
		Integer b1 = new Integer(1);
		Integer b2 = new Integer(1);

		Assertion.assertNotSame(b1, b2);
		try {
			Assertion.assertNotSame(b1, b1);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
		try {
			Assertion.assertSame(b1, b2);
			fail("An AssertionFailedException should have been thrown");
		} catch (AssertionFailedException afe) {
		}
	}

	// Test that an assertion message is correctly formatted.
	// We want the message to be trimmed and to end with a single period.
	public void testAssertionMessage() {
		try {
			Assertion.assertEquals("foo ", 1, 2);
		} catch (AssertionFailedException afe) {
			String msg = afe.getMessage();
			assertTrue(msg.startsWith("foo. "));
		}
		try {
			Assertion.assertEquals("foo.", 1, 2);
		} catch (AssertionFailedException afe) {
			String msg = afe.getMessage();
			assertTrue(msg.startsWith("foo. "));
		}
		try {
			Assertion.assertEquals(null, 1, 2);
		} catch (AssertionFailedException afe) {
		}
	}
	
	public void runTest() throws Throwable {
		if (getName().equals("testFail")) {
			testFail();
		} else if (getName().equals("testFailMessage")) {
			testFailMessage();
		} else if (getName().equals("testFailBoolean")) {
			testFailBoolean();
		} else if (getName().equals("testAssertEqualsStrings")) {
			testAssertEqualsStrings();
		} else if (getName().equals("testAssertEqualsBoolean")) {
			testAssertEqualsBoolean();
		} else if (getName().equals("testAssertEqualsFloat")) {
			testAssertEqualsFloat();
		} else if (getName().equals("testAssertEqualsDouble")) {
			testAssertEqualsDouble();
		} else if (getName().equals("testAssertEqualsBytes")) {
			testAssertEqualsBytes();
		} else if (getName().equals("testAssertEqualsShorts")) {
			testAssertEqualsShorts();
		} else if (getName().equals("testAssertEqualsInts")) {
			testAssertEqualsInts();
		} else if (getName().equals("testAssertEqualsLongs")) {
			testAssertEqualsLongs();
		} else if (getName().equals("testAssertEqualsObjects")) {
			testAssertEqualsObjects();
		} else if (getName().equals("testAssertSame")) {
			testAssertSame();
		} else if (getName().equals("testAssertionMessage")) {
			testAssertionMessage();
		} else {
			fail("Unknown test: " + getName());
		}
	}

	public Test suite() {
		TestSuite suite = new TestSuite(this.getClass(), new String[] {
				"testFail", "testFailMessage", "testFailBoolean",
				"testAssertEqualsStrings", "testAssertEqualsBoolean",
				"testAssertEqualsFloat", "testAssertEqualsDouble",
				"testAssertEqualsBytes", "testAssertEqualsShorts",
				"testAssertEqualsInts", "testAssertEqualsLongs",
				"testAssertEqualsObjects", "testAssertSame",
				"testAssertionMessage" });
		return suite;
	}
}
