package jmunit.framework.cldc11;

import j2meunit.framework.Test;
import j2meunit.framework.TestCase;
import j2meunit.framework.TestSuite;
import jmunit.framework.cldc11.mocks.MockTestListener;

import com.hammingweight.hammock.Hammock;

public class TestResultTest extends TestCase {

	// The OUT.
	private TestResult testResult;

	// The mocks.
	private Hammock hammock;

	private MockTestListener mockListener1;

	private MockTestListener mockListener2;

	public void setUp() {
		// Create the mocks.
		this.hammock = new Hammock();
		this.mockListener1 = new MockTestListener(this.hammock);
		this.mockListener2 = new MockTestListener(this.hammock);

		// Create the OUT.
		this.testResult = new TestResult();
		this.testResult.removeListeners();
		this.testResult.addListener(this.mockListener1);
	}

	// If the TestResult is told that a test has started, a TestListener
	// should be told as well.
	public void testStartTest() throws Throwable {
		// Expectations.
		this.hammock.setExpectation(
				MockTestListener.MTHD_START_TEST_$_CLASS_STRING, new Object[] {
						null, "footest" });

		// Exercise.
		this.testResult.startTest(null, "footest");

		// Verify.
		this.hammock.verify();
	}

	// If the TestResult is told that a test has started, TestListeners
	// should be told as well.
	public void testStartTestTwoListeners() throws Throwable {
		// Expectations.
		this.hammock.setExpectation(
				MockTestListener.MTHD_START_TEST_$_CLASS_STRING,
				this.mockListener1, new Object[] { null, "footest" });
		this.hammock.setExpectation(
				MockTestListener.MTHD_START_TEST_$_CLASS_STRING,
				this.mockListener2, new Object[] { null, "footest" });

		// Exercise.
		this.testResult.addListener(this.mockListener2);
		this.testResult.startTest(null, "footest");

		// Verify.
		this.hammock.verify();
	}

	// If the TestResult is told that a test has ended, a TestListener
	// should be told as well.
	public void testEndTest() throws Throwable {
		// Expectations.
		this.hammock.setExpectation(
				MockTestListener.MTHD_END_TEST_$_CLASS_STRING, new Object[] {
						null, "footest" });

		// Exercise.
		this.testResult.endTest(null, "footest");

		// Verify.
		this.hammock.verify();
	}

	// Check that if a listener throws an Exception that other listeners still
	// get notified.
	public void testStartException() throws Throwable {
		// Expectations.
		this.hammock.setExpectation(
				MockTestListener.MTHD_START_TEST_$_CLASS_STRING,
				this.mockListener1, new Object[] { null, "footest" })
				.setThrowable(new RuntimeException());
		this.hammock.setExpectation(
				MockTestListener.MTHD_START_TEST_$_CLASS_STRING,
				this.mockListener2, new Object[] { null, "footest" });

		// Exercise.
		this.testResult.addListener(this.mockListener2);
		this.testResult.startTest(null, "footest");

		// Verify.
		this.hammock.verify();
	}

	// Check that if a listener throws an Exception that other listeners still
	// get notified.
	public void testEndException() throws Throwable {
		// Expectations.
		this.hammock.setExpectation(
				MockTestListener.MTHD_END_TEST_$_CLASS_STRING,
				this.mockListener1, new Object[] { null, "footest" })
				.setThrowable(new RuntimeException());
		this.hammock.setExpectation(
				MockTestListener.MTHD_END_TEST_$_CLASS_STRING,
				this.mockListener2, new Object[] { null, "footest" });

		// Exercise.
		this.testResult.addListener(this.mockListener2);
		this.testResult.endTest(null, "footest");

		// Verify.
		this.hammock.verify();
	}

	// If the TestResult is told that a test has ended, TestListeners
	// should be told as well.
	public void testEndTestTwoListeners() throws Throwable {
		// Expectations.
		this.hammock.setExpectation(
				MockTestListener.MTHD_END_TEST_$_CLASS_STRING,
				this.mockListener1, new Object[] { null, "footest" });
		this.hammock.setExpectation(
				MockTestListener.MTHD_END_TEST_$_CLASS_STRING,
				this.mockListener2, new Object[] { null, "footest" });

		// Exercise.
		this.testResult.addListener(this.mockListener2);
		this.testResult.endTest(null, "footest");

		// Verify.
		this.hammock.verify();
	}

	// If an Exception other than an AssertionFailedException is thrown, then
	// the addError method of the TestListener class must be called.
	public void testAddError() throws Throwable {
		// Expectations.
		Exception e = new Exception();
		this.hammock.setExpectation(
				MockTestListener.MTHD_ADD_ERROR_$_CLASS_STRING_THROWABLE,
				new Object[] { null, "bartest", e });

		// Exercise.
		this.testResult.addError(null, "bartest", e);

		// Verify.
		this.hammock.verify();
	}

	// If an AssertionFailedException is thrown, then
	// the addFailure method of the TestListener class must be called.
	public void testAddFailure() throws Throwable {
		// Expectations.
		AssertionFailedException e = new AssertionFailedException();
		this.hammock
				.setExpectation(
						MockTestListener.MTHD_ADD_FAILURE_$_CLASS_STRING_ASSERTIONFAILEDEXCEPTION,
						new Object[] { null, "bartest", e });

		// Exercise.
		this.testResult.addError(null, "bartest", e);

		// Verify.
		this.hammock.verify();
	}

	// If a test reports two errors, the TestResult should only notify the
	// listeners of the first error.
	public void testTwoErrorsOneTest() throws Throwable {
		// Expectations.
		this.hammock
				.setExpectation(MockTestListener.MTHD_START_TEST_$_CLASS_STRING);
		this.hammock
				.setExpectation(MockTestListener.MTHD_ADD_ERROR_$_CLASS_STRING_THROWABLE);
		this.hammock
				.setExpectation(MockTestListener.MTHD_END_TEST_$_CLASS_STRING);
		this.hammock.setStrictOrdering();

		// Exercise. We report two exceptions.
		this.testResult.startTest(null, "foo");
		this.testResult.addError(null, "foo", new RuntimeException());
		this.testResult.addError(null, "foo", new Error());
		this.testResult.endTest(null, "foo");

		// Verify.
		this.hammock.verify();
	}

	// If two tests report errors, both errors must be passed to the listeners.
	public void testTwoErrorsTwoTests() throws Throwable {
		// Expectations.
		this.hammock
				.setExpectation(MockTestListener.MTHD_START_TEST_$_CLASS_STRING);
		this.hammock
				.setExpectation(MockTestListener.MTHD_ADD_ERROR_$_CLASS_STRING_THROWABLE);
		this.hammock
				.setExpectation(MockTestListener.MTHD_END_TEST_$_CLASS_STRING);
		this.hammock
				.setExpectation(MockTestListener.MTHD_START_TEST_$_CLASS_STRING);
		this.hammock
				.setExpectation(MockTestListener.MTHD_ADD_ERROR_$_CLASS_STRING_THROWABLE);
		this.hammock
				.setExpectation(MockTestListener.MTHD_END_TEST_$_CLASS_STRING);
		this.hammock.setStrictOrdering();

		// Exercise. We report two exceptions.
		this.testResult.startTest(null, "foo");
		this.testResult.addError(null, "foo", new RuntimeException());
		this.testResult.endTest(null, "foo");
		this.testResult.startTest(null, "bar");
		this.testResult.addError(null, "bar", new Error());
		this.testResult.endTest(null, "bar");

		// Verify.
		this.hammock.verify();
	}

	// If a test is cleared, the listener must be told.
	public void testClear() throws Throwable {
		// Expectations.
		this.hammock.setExpectation(MockTestListener.MTHD_CLEAR,
				this.mockListener1, new Object[] {});
		this.hammock.setExpectation(MockTestListener.MTHD_CLEAR,
				this.mockListener2, new Object[] {});

		// Exercise.
		this.testResult.addListener(this.mockListener2);
		this.testResult.clear();

		// Verify.
		this.hammock.verify();
	}

	public void runTest() throws Throwable {
		if (getName().equals("testStartTest")) {
			testStartTest();
		} else if (getName().equals("testStartTest2")) {
			testStartTestTwoListeners();
		} else if (getName().equals("testStartException")) {
			testStartException();
		} else if (getName().equals("testEndTest")) {
			testEndTest();
		} else if (getName().equals("testEndTest2")) {
			testEndTestTwoListeners();
		} else if (getName().equals("testEndException")) {
			testEndException();
		} else if (getName().equals("testAddError")) {
			testAddError();
		} else if (getName().equals("testAddFailure")) {
			testAddFailure();
		} else if (getName().equals("testTwoErrorsOneTest")) {
			testTwoErrorsOneTest();
		} else if (getName().equals("testTwoErrorsTwoTests")) {
			testTwoErrorsTwoTests();
		} else if (getName().equals("testClear")) {
			testClear();
		} else {
			fail("Unknown test: " + getName());
		}
	}

	public Test suite() {
		TestSuite suite = new TestSuite(this.getClass(), new String[] {
				"testStartTest", "testStartTest2", "testStartException",
				"testEndTest", "testEndTest2", "testEndException",
				"testAddError", "testAddFailure", "testTwoErrorsOneTest",
				"testTwoErrorsTwoTests", "testClear" });
		return suite;
	}
}
